#pragma once

// std c++ headers
#include <string>

namespace AMDiS
{
  // forward declarations
  class AdaptInfo;
  class ProblemIterationInterface;
  class ProblemTimeInterface;

  /// Interface for adaption loops.
  class AdaptBase
  {
  public:
    /// Constructor
    AdaptBase(std::string sname,
              ProblemIterationInterface* problemIteration,
              AdaptInfo& adapt,
              ProblemTimeInterface* problemTime = nullptr,
              AdaptInfo* initialAdaptInfo = nullptr)
      : name(sname)
      , problemIteration(problemIteration)
      , adaptInfo(adapt)
      , problemTime(problemTime)
      , initialAdaptInfo(initialAdaptInfo)
    {}

    /// Destructor
    virtual ~AdaptBase() {}

    /** \brief
     * Pure virtual method. Must be overloaded by sub classes to perform
     * a concrete adaption loop.
     */
    virtual int adapt() = 0;

    /// Returns \ref name
    std::string getName() const
    {
      return name;
    }

    /// Returns \ref problemIteration
    ProblemIterationInterface* getProblemIteration() const
    {
      return problemIteration;
    }

    ///
    void setProblemIteration(ProblemIterationInterface* pii)
    {
      problemIteration = pii;
    }

    /// Returns \ref adaptInfo
    AdaptInfo& getAdaptInfo() const
    {
      return adaptInfo;
    }

    /// Returns \ref problemTime
    ProblemTimeInterface* getProblemTime() const
    {
      return problemTime;
    }

    ///
    void setProblemTime(ProblemTimeInterface* pti)
    {
      problemTime = pti;
    }

    /// Returns \ref initialAdaptInfo
    AdaptInfo& getInitialAdaptInfo() const
    {
      return *initialAdaptInfo;
    }

  protected:
    /// Name of the adaption loop
    std::string name;

    /// Problem iteration interface
    ProblemIterationInterface* problemIteration;

    /// Main adapt info
    AdaptInfo& adaptInfo;

    /// problem time interface
    ProblemTimeInterface* problemTime;

    /** \brief
     * Adapt info for initial adapt. Will be given to
     * problemTime->solveInitialProblem().
     */
    AdaptInfo* initialAdaptInfo;

    /// Info level
    static int info;
  };

} // end namespace AMDiS
