#pragma once

#include <type_traits>

#include <amdis/GridFunctionOperator.hpp>
#include <amdis/common/ValueCategory.hpp>

namespace AMDiS
{
  /**
   * \addtogroup operators
   * @{
   **/

  namespace tag
  {
    struct test {};
  }


  /// zero-order vector-operator \f$ (c\, \psi) \f$
  template <class LocalContext, class GridFct>
  class GridFunctionOperator<tag::test, LocalContext, GridFct>
      : public GridFunctionOperatorBase<GridFunctionOperator<tag::test, LocalContext, GridFct>,
                                        LocalContext, GridFct>
  {
    using Self = GridFunctionOperator;
    using Super = GridFunctionOperatorBase<Self, LocalContext, GridFct>;

    static_assert( Category::Scalar<typename GridFct::Range>, "Expression must be of scalar type." );

  public:
    GridFunctionOperator(tag::test, GridFct const& expr)
      : Super(expr, 0)
    {}

    template <class Context, class Node, class ElementVector>
    void getElementVector(Context const& context,
                          Node const& node,
                          ElementVector& elementVector)
    {
      static_assert(Node::isLeaf, "Operator can be applied to Leaf-Nodes only");

      auto const& localFE = node.finiteElement();

      using LocalBasisType = typename std::decay_t<decltype(localFE)>::Traits::LocalBasisType;
      using RangeType = typename LocalBasisType::Traits::RangeType;

      auto const& quad = this->getQuadratureRule(context.type(), node);
      for (auto const& qp : quad) {
        // Position of the current quadrature point in the reference element
        decltype(auto) local = context.local(qp.position());

        // The multiplicative factor in the integral transformation formula
        const auto factor = Super::coefficient(local) * context.integrationElement(qp.position()) * qp.weight();

        thread_local std::vector<RangeType> shapeValues;
        localFE.localBasis().evaluateFunction(local, shapeValues);

        for (std::size_t i = 0; i < localFE.size(); ++i) {
          const auto local_i = node.localIndex(i);
          elementVector[local_i] += factor * shapeValues[i];
        }
      }
    }
  };

  /** @} **/

} // end namespace AMDiS
