#pragma once

namespace AMDiS
{
  /// Interface for Preconditioner types
  template <class Matrix, class VectorX, class VectorB = VectorX>
  struct PreconditionerInterface
  {
    /// Virtual destructor.
    virtual ~PreconditionerInterface() = default;

    /// Is called a the beginning of a solution procedure
    virtual void init(Matrix const& fullMatrix) = 0;

    /// Is called at the end of a solution procedure
    virtual void exit() = 0;

    /// Apply the preconditioner to a vector \p b and store the result in \p x
    virtual void solve(VectorB const& b, VectorX& x) const = 0;

    /// Apply the transposed preconditioner to a vector \p b and store the result in \p x
    virtual void adjoint_solve(VectorB const& b, VectorX& x) const
    {
      error_exit("Must be implemented by derived class.");
    }
  };

} // end namespace AMDiS
