#pragma once

#include <dune/istl/bvector.hh>

#include <amdis/Output.hpp>
#include <amdis/linear_algebra/DOFVectorBase.hpp>

namespace AMDiS
{
  template <class T, class = void>
  struct BlockVectorType
  {
    using type = Dune::FieldVector<T,1>;
  };

  template <class T>
  struct BlockVectorType<T, typename T::field_type>
  {
    using type = T;
  };

	template <class ValueType>
	class IstlVector
	{
	public:
    /// The type of the elements of the DOFVector
	  using value_type = typename BlockVectorType<ValueType>::type;

    /// The vector type of the underlying base vector
	  using BaseVector = Dune::BlockVector<value_type>;

    /// The index/size - type
	  using size_type  = typename BaseVector::size_type;

  public:
    /// Constructor. Constructs new BaseVector.
    IstlVector() = default;

    /// Return the data-vector \ref vector
    BaseVector const& vector() const
    {
      return vector_;
    }

    /// Return the data-vector \ref vector
    BaseVector& vector()
    {
      return vector_;
    }


    /// Return the current size of the \ref vector_
    size_type size() const
    {
      return vector_.size();
    }

    /// Resize the \ref vector_ to the size \p s
    void resize(size_type s)
    {
      vector_.resize(s);
    }


    /// Access the entry \p i of the \ref vector with read-access.
    value_type const& operator[](size_type i) const
    {
      test_exit_dbg(i < vector_.size(),
        "Index {} out of range [0,{})", i, vector_.size());
      return vector_[i];
    }

    /// Access the entry \p i of the \ref vector with write-access.
    value_type& operator[](size_type i)
    {
      test_exit_dbg(i < vector_.size(),
        "Index {} out of range [0,{})", i, vector_.size());
      return vector_[i];
    }

	private:
    BaseVector vector_;
	};


  template <class BasisType, class VectorType = double>
  using DOFVector = DOFVectorBase<BasisType, IstlVector<VectorType>>;

  /// Constructor a dofvector from given basis and name
  template <class ValueType = double, class Basis>
  DOFVector<Basis, ValueType>
  makeDOFVector(Basis const& basis)
  {
    return {basis};
  }

} // end namespace AMDiS
